@@ -103,9 +103,12 @@ def configure(repl):
     # Use this colorscheme for the code.
     repl.use_code_colorscheme('pastie')
 
-    # Enable 24bit True color. (Not all terminals support this. -- maybe check
-    # $TERM before changing.)
-    repl.true_color = False
+    # Set color depth (keep in mind that not all terminals support true color).
+
+    #repl.color_depth = 'DEPTH_1_BIT'  # Monochrome.
+    #repl.color_depth = 'DEPTH_4_BIT'  # ANSI colors only.
+    repl.color_depth = 'DEPTH_8_BIT'  # The default, 256 colors.
+    #repl.color_depth = 'DEPTH_24_BIT'  # True color.
 
     # Syntax.
     repl.enable_syntax_highlighting = True
