@@ -10,19 +10,19 @@ Utility for creating a Python repl.
 from __future__ import unicode_literals
 
 from pygments.lexers import PythonTracebackLexer, PythonLexer
-from pygments.styles.default import DefaultStyle
+from pygments.token import Token
 
-from prompt_toolkit.application import AbortAction
-from prompt_toolkit.enums import DEFAULT_BUFFER
-from prompt_toolkit.interface import AcceptAction
-from prompt_toolkit.key_binding.vi_state import InputMode
-from prompt_toolkit.layout.utils import token_list_width
-from prompt_toolkit.shortcuts import create_asyncio_eventloop
-from prompt_toolkit.styles import style_from_pygments
+from prompt_toolkit.document import Document
+from prompt_toolkit.eventloop.defaults import use_asyncio_event_loop
+from prompt_toolkit.formatted_text import merge_formatted_text, FormattedText
+from prompt_toolkit.formatted_text.utils import fragment_list_width
 from prompt_toolkit.utils import DummyContext
+from prompt_toolkit.shortcuts import set_title, clear_title
+from prompt_toolkit.shortcuts import print_formatted_text
+from prompt_toolkit.formatted_text import PygmentsTokens
 
-from .python_input import PythonInput, PythonCommandLineInterface
-from .eventloop import create_eventloop
+from .python_input import PythonInput
+from .eventloop import inputhook
 
 import os
 import six
@@ -41,62 +41,68 @@ __all__ = (
 class PythonRepl(PythonInput):
     def __init__(self, *a, **kw):
         self._startup_paths = kw.pop('startup_paths', None)
-
-        kw.update({
-            '_accept_action': AcceptAction.run_in_terminal(
-                handler=self._process_document, render_cli_done=True),
-            '_on_start': self._on_start,
-            '_on_exit': AbortAction.RETURN_NONE,
-        })
-
         super(PythonRepl, self).__init__(*a, **kw)
+        self._load_start_paths()
 
-    def _on_start(self, cli):
-        """
-        Start the Read-Eval-Print Loop.
-        """
+    def _load_start_paths(self):
+        " Start the Read-Eval-Print Loop. "
         if self._startup_paths:
             for path in self._startup_paths:
                 if os.path.exists(path):
-                    with open(path, 'r') as f:
+                    with open(path, 'rb') as f:
                         code = compile(f.read(), path, 'exec')
                         six.exec_(code, self.get_globals(), self.get_locals())
                 else:
-                    output = cli.output
+                    output = self.app.output
                     output.write('WARNING | File not found: {}\n\n'.format(path))
 
-    def _process_document(self, cli, buffer):
-        line = buffer.text
+    def run(self):
+        if self.terminal_title:
+            set_title(self.terminal_title)
+
+        while True:
+            # Run the UI.
+            try:
+                text = self.app.run(inputhook=inputhook)
+            except EOFError:
+                return
+            except KeyboardInterrupt:
+                # Abort - try again.
+                self.default_buffer.document = Document()
+            else:
+                self._process_text(text)
+
+        if self.terminal_title:
+            clear_title()
+
+    def _process_text(self, text):
+        line = self.default_buffer.text
 
         if line and not line.isspace():
             try:
                 # Eval and print.
-                self._execute(cli, line)
+                self._execute(line)
             except KeyboardInterrupt as e:  # KeyboardInterrupt doesn't inherit from Exception.
-                self._handle_keyboard_interrupt(cli, e)
+                self._handle_keyboard_interrupt(e)
             except Exception as e:
-                self._handle_exception(cli, e)
+                self._handle_exception(e)
 
             if self.insert_blank_line_after_output:
-                cli.output.write('\n')
+                self.app.output.write('\n')
 
             self.current_statement_index += 1
             self.signatures = []
 
-            # Append to history and reset.
-            cli.search_state.text = ''
-            cli.buffers[DEFAULT_BUFFER].reset(append_to_history=True)
-
-        # Make sure that we end up in insert mode.
-        # (Not exactly the right place to check this.)
-        if cli.vi_state.input_mode == InputMode.NAVIGATION:
-            cli.vi_state.input_mode = InputMode.INSERT
-
-    def _execute(self, cli, line):
+    def _execute(self, line):
         """
         Evaluate the line and print the result.
         """
-        output = cli.output
+        output = self.app.output
+
+        # WORKAROUND: Due to a bug in Jedi, the current directory is removed
+        # from sys.path. See: https://github.com/davidhalter/jedi/issues/1148
+        if '' not in sys.path:
+            sys.path.insert(0, '')
 
         def compile_with_flags(code, mode):
             " Compile code with the right compiler flags. "
@@ -106,7 +112,7 @@ class PythonRepl(PythonInput):
 
         if line.lstrip().startswith('\x1a'):
             # When the input starts with Ctrl-Z, quit the REPL.
-            cli.exit()
+            self.app.exit()
 
         elif line.lstrip().startswith('!'):
             # Run as shell command
@@ -121,7 +127,7 @@ class PythonRepl(PythonInput):
                 locals['_'] = locals['_%i' % self.current_statement_index] = result
 
                 if result is not None:
-                    out_tokens = self.get_output_prompt_tokens(cli)
+                    out_prompt = self.get_output_prompt()
 
                     try:
                         result_str = '%r\n' % (result, )
@@ -133,12 +139,24 @@ class PythonRepl(PythonInput):
                         result_str = '%s\n' % repr(result).decode('utf-8')
 
                     # Align every line to the first one.
-                    line_sep = '\n' + ' ' * token_list_width(out_tokens)
+                    line_sep = '\n' + ' ' * fragment_list_width(out_prompt)
                     result_str = line_sep.join(result_str.splitlines()) + '\n'
 
                     # Write output tokens.
-                    out_tokens.extend(_lex_python_result(result_str))
-                    cli.print_tokens(out_tokens)
+                    if self.enable_syntax_highlighting:
+                        formatted_output = merge_formatted_text([
+                            out_prompt,
+                            PygmentsTokens(list(_lex_python_result(result_str))),
+                        ])
+                    else:
+                        formatted_output = FormattedText(
+                                out_prompt + [('', result_str)])
+
+                    print_formatted_text(
+                        formatted_output, style=self._current_style,
+                        style_transformation=self.style_transformation,
+                        include_default_pygments_style=False)
+
             # If not a valid `eval` expression, run using `exec` instead.
             except SyntaxError:
                 code = compile_with_flags(line, 'exec')
@@ -146,13 +164,16 @@ class PythonRepl(PythonInput):
 
             output.flush()
 
-    @classmethod
-    def _handle_exception(cls, cli, e):
-        output = cli.output
+    def _handle_exception(self, e):
+        output = self.app.output
 
         # Instead of just calling ``traceback.format_exc``, we take the
         # traceback and skip the bottom calls of this framework.
         t, v, tb = sys.exc_info()
+
+        # Required for pdb.post_mortem() to work.
+        sys.last_type, sys.last_value, sys.last_traceback = t, v, tb
+
         tblist = traceback.extract_tb(tb)
 
         for line_nr, tb_tuple in enumerate(tblist):
@@ -175,15 +196,21 @@ class PythonRepl(PythonInput):
         # Format exception and write to output.
         # (We use the default style. Most other styles result
         # in unreadable colors for the traceback.)
-        tokens = _lex_python_traceback(tb)
-        cli.print_tokens(tokens, style=style_from_pygments(DefaultStyle))
+        if self.enable_syntax_highlighting:
+            tokens = list(_lex_python_traceback(tb))
+        else:
+            tokens = [(Token, tb)]
+
+        print_formatted_text(
+            PygmentsTokens(tokens), style=self._current_style,
+            style_transformation=self.style_transformation,
+            include_default_pygments_style=False)
 
         output.write('%s\n' % e)
         output.flush()
 
-    @classmethod
-    def _handle_keyboard_interrupt(cls, cli, e):
-        output = cli.output
+    def _handle_keyboard_interrupt(self, e):
+        output = self.app.output
 
         output.write('\rKeyboardInterrupt\n\n')
         output.flush()
@@ -239,7 +266,7 @@ def run_config(repl, config_file='~/.ptpython/config.py'):
     try:
         namespace = {}
 
-        with open(config_file, 'r') as f:
+        with open(config_file, 'rb') as f:
             code = compile(f.read(), config_file, 'exec')
             six.exec_(code, namespace, namespace)
 
@@ -289,14 +316,11 @@ def embed(globals=None, locals=None, configure=None,
 
     # Create eventloop.
     if return_asyncio_coroutine:
-        eventloop = create_asyncio_eventloop()
-    else:
-        eventloop = create_eventloop()
+        use_asyncio_event_loop()
 
     # Create REPL.
-    repl = PythonRepl(get_globals, get_locals, vi_mode=vi_mode,
-                      history_filename=history_filename,
-                      startup_paths=startup_paths)
+    repl = PythonRepl(get_globals=get_globals, get_locals=get_locals, vi_mode=vi_mode,
+                      history_filename=history_filename, startup_paths=startup_paths)
 
     if title:
         repl.terminal_title = title
@@ -304,17 +328,17 @@ def embed(globals=None, locals=None, configure=None,
     if configure:
         configure(repl)
 
-    cli = PythonCommandLineInterface(python_input=repl, eventloop=eventloop)
+    app = repl.app
 
     # Start repl.
-    patch_context = cli.patch_stdout_context() if patch_stdout else DummyContext()
+    patch_context = app.patch_stdout_context() if patch_stdout else DummyContext()
 
-    if return_asyncio_coroutine:
+    if return_asyncio_coroutine: # XXX
         def coroutine():
             with patch_context:
-                for future in cli.run_async():
+                for future in app.run_async():
                     yield future
         return coroutine()
     else:
         with patch_context:
-            cli.run()
+            repl.run()
